package com.xhkjedu.sexam.service.report;

import com.xhkjedu.sexam.mapper.exam.EBaseMapper;
import com.xhkjedu.sexam.mapper.exam.EClassMapper;
import com.xhkjedu.sexam.mapper.exam.ESubjectMapper;
import com.xhkjedu.sexam.mapper.paperstudent.EPaperStudentMapper;
import com.xhkjedu.sexam.mapper.report.ERbaseMapper;
import com.xhkjedu.sexam.mapper.reportstu.ERstudentMapper;
import com.xhkjedu.sexam.model.reportstu.ERstudent;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.DoubleSummaryStatistics;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ywx
 * @classname EReportGenerateService
 * @description 考试报告生成
 * @date 2022/8/8 10:04
 **/
@Service
public class EReportGenerateService {
    @Resource
    private EBaseMapper eBaseMapper;
    @Resource
    private EClassMapper eClassMapper;
    @Resource
    private ESubjectMapper eSubjectMapper;
    @Resource
    private EPaperStudentMapper ePaperStudentMapper;

    @Resource
    private ERbaseMapper eRbaseMapper;
    @Resource
    private ERstudentMapper eRstudentMapper;

    //生成考试报告
    public void generateExamReport(Integer examid) {
        List<Map> classes = eClassMapper.listByExamId(examid);
        List<Map> subjects = eSubjectMapper.listSubject(examid);
        int classnum = classes.size();
        int stunum = classes.stream().mapToInt(m -> (int) m.get("classnum")).sum();
        int subjectnum = subjects.size();
        /*ERbase rbase = new ERbase();
        rbase.setExamid(examid);
        rbase.setClassnum(classnum);
        rbase.setStunum(stunum);
        rbase.setSubjectnum(subjectnum);
        eRbaseMapper.insertSelective(rbase);*/

        String subjectid = "zf";
        String subjectname = "总分";
        Map<String, String> subjectMap = subjects.stream().collect(Collectors.toMap(m -> m.get("subjectid").toString(),
                m -> m.get("subjectname").toString()));

        List<ERstudent> schoolStudents = new ArrayList<>();//学校学生成绩
        List<ERstudent> classStudents = new ArrayList<>();//班级学生成绩
        List<ERstudent> subjectStudents = new ArrayList<>();//科目学生成绩、
        Map<String, Integer> schoolRank = new HashMap<>();//学校分数排名
        Map<String, Integer> classRank = new HashMap<>();//班级分数排名
        Map<String, Integer> ssubjectRank = new HashMap<>();//学校科目分数排名
        Map<String, Integer> csubjectRank = new HashMap<>();//班级科目分数排名

        List<ERstudent> students = ePaperStudentMapper.listByExamId(examid);
        Map<Integer, List<ERstudent>> schoolCollect = students.stream().collect(Collectors.groupingBy(s -> s.getStudentid()));
        for (Map.Entry<Integer, List<ERstudent>> entry : schoolCollect.entrySet()) {
            Integer studentid = entry.getKey();
            ERstudent cs = new ERstudent();
            cs.setStudentid(studentid);
            cs.setScore(entry.getValue().stream().mapToDouble(s -> s.getScore()).sum());
            schoolStudents.add(cs);
        }
        setRank(schoolStudents, schoolRank, 1);//学校分数排名

        Map<Integer, List<ERstudent>> cCollect = students.stream().collect(Collectors.groupingBy(s -> s.getClassid()));
        for (Map.Entry<Integer, List<ERstudent>> entry : cCollect.entrySet()) {
            Integer classid = entry.getKey();
            List<ERstudent> studentList = entry.getValue();
            Map<Integer, List<ERstudent>> sCollect = studentList.stream().collect(Collectors.groupingBy(s -> s.getStudentid()));
            for (Map.Entry<Integer, List<ERstudent>> csEntry : sCollect.entrySet()) {
                ERstudent cs = new ERstudent();
                cs.setClassid(classid);
                cs.setStudentid(csEntry.getKey());
                cs.setSubjectid(subjectid);
                cs.setSubjectname(subjectname);
                cs.setScore(csEntry.getValue().stream().mapToDouble(s -> s.getScore()).sum());
                classStudents.add(cs);
            }
            setRank(classStudents, classRank, 2);//班级分数排名

            Map<String, List<ERstudent>> subCollect = studentList.stream().collect(Collectors.groupingBy(s -> s.getSubjectid()));
            for (Map.Entry<String, List<ERstudent>> ssEntry : subCollect.entrySet()) {
                subjectid = ssEntry.getKey();
                subjectname = subjectMap.get(subjectid);
                studentList = ssEntry.getValue();
                for (ERstudent cs : studentList) {
                    cs.setSubjectname(subjectname);
                    subjectStudents.add(cs);
                }
            }
            setRank(subjectStudents, ssubjectRank, 3);//科目分数排名
        }

        DoubleSummaryStatistics schoolStatistics = schoolStudents.stream().mapToDouble(s -> s.getScore()).summaryStatistics();
        DoubleSummaryStatistics classStatistics = classStudents.stream().mapToDouble(s -> s.getScore()).summaryStatistics();
        for (ERstudent cs : classStudents) {
            Integer studentid = cs.getStudentid();
            cs.setClassrank(classRank.get(cs.getClassid()+"_"+ studentid));
            cs.setClassmaxscore(classStatistics.getMax());
            cs.setClassavgscore(classStatistics.getAverage());
            cs.setSchoolrank(schoolRank.get(studentid));
            cs.setSchoolmaxscore(schoolStatistics.getMax());
            cs.setSchoolavgscore(schoolStatistics.getAverage());
        }
        eRstudentMapper.insertList(classStudents);

        for (ERstudent cs : subjectStudents) {
            Integer studentid = cs.getStudentid();
            cs.setClassrank(classRank.get(cs.getClassid()+"_"+ studentid));
            cs.setClassmaxscore(classStatistics.getMax());
            cs.setClassavgscore(classStatistics.getAverage());
            cs.setSchoolrank(schoolRank.get(studentid));
            cs.setSchoolmaxscore(schoolStatistics.getMax());
            cs.setSchoolavgscore(schoolStatistics.getAverage());
        }
        eRstudentMapper.insertList(subjectStudents);
    }

    //分数排名
    private void setRank(List<ERstudent> students, Map<String, Integer> rank, Integer code) {
        students =
                students.stream().sorted((s1, s2) -> -Double.compare(s1.getScore(), s2.getScore())).collect(Collectors.toList());
        List<Map.Entry<Double, List<ERstudent>>> gslist2 = students.stream()
                .collect(Collectors.groupingBy(ERstudent::getScore)).entrySet()
                .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
        Integer index = 1;
        String key;
        for (Map.Entry<Double, List<ERstudent>> entry : gslist2) {
            for (ERstudent cgs : entry.getValue()) {
                Integer studentid = cgs.getStudentid();
                Integer classid = cgs.getClassid();
                String subjectid = cgs.getSubjectid();
                if (code == 1) {//学校
                    key = studentid.toString();
                } else if (code == 2) {//班级
                    key = classid + "_" + studentid;
                } else {//科目
                    key = subjectid + "_" + studentid;
                }
                rank.put(key, index);
            }
            index = index + entry.getValue().size();
        }
    }
}
